package cn.ibizlab.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.sample.domain.DYNADASHBOARD;
import cn.ibizlab.core.sample.filter.DYNADASHBOARDSearchContext;
import cn.ibizlab.core.sample.service.IDYNADASHBOARDService;

import cn.ibizlab.util.helper.CachedBeanCopier;
import cn.ibizlab.util.helper.DEFieldCacheMap;
import com.alibaba.fastjson.JSONObject;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.sample.mapper.DYNADASHBOARDMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import org.springframework.util.StringUtils;

/**
 * 实体[动态数据看板] 服务对象接口实现
 */
@Slf4j
@Service("DYNADASHBOARDServiceImpl")
public class DYNADASHBOARDServiceImpl extends ServiceImpl<DYNADASHBOARDMapper, DYNADASHBOARD> implements IDYNADASHBOARDService {


    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(DYNADASHBOARD et) {
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getDynadashboardid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<DYNADASHBOARD> list) {
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(DYNADASHBOARD et) {
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("dynadashboardid", et.getDynadashboardid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getDynadashboardid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<DYNADASHBOARD> list) {
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public DYNADASHBOARD get(String key) {
        DYNADASHBOARD et = getById(key);
        if (et == null) {
            throw new BadRequestAlertException("数据不存在", this.getClass().getSimpleName(), key);
        }
        else {
        }
        return et;
    }

    @Override
    public DYNADASHBOARD getDraft(DYNADASHBOARD et) {
        return et;
    }

    @Override
    public boolean checkKey(DYNADASHBOARD et) {
        return (!ObjectUtils.isEmpty(et.getDynadashboardid())) && (!Objects.isNull(this.getById(et.getDynadashboardid())));
    }
    @Override
    @Transactional
    public boolean save(DYNADASHBOARD et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(DYNADASHBOARD et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? getProxyService().update(et) : getProxyService().create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<DYNADASHBOARD> list) {
        List<DYNADASHBOARD> create = new ArrayList<>();
        List<DYNADASHBOARD> update = new ArrayList<>();
        for (DYNADASHBOARD et : list) {
            if (ObjectUtils.isEmpty(et.getDynadashboardid()) || ObjectUtils.isEmpty(getById(et.getDynadashboardid()))) {
                create.add(et);
            } else {
                update.add(et);
            }
        }
        if (create.size() > 0) {
            getProxyService().createBatch(create);
        }
        if (update.size() > 0) {
            getProxyService().updateBatch(update);
        }
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<DYNADASHBOARD> list) {
        List<DYNADASHBOARD> create = new ArrayList<>();
        List<DYNADASHBOARD> update = new ArrayList<>();
        for (DYNADASHBOARD et : list) {
            if (ObjectUtils.isEmpty(et.getDynadashboardid()) || ObjectUtils.isEmpty(getById(et.getDynadashboardid()))) {
                create.add(et);
            } else {
                update.add(et);
            }
        }
        if (create.size() > 0) {
            getProxyService().createBatch(create);
        }
        if (update.size() > 0) {
            getProxyService().updateBatch(update);
        }
    }



    /**
     * 查询集合 数据集
     */
    @Override
    public Page<DYNADASHBOARD> searchDefault(DYNADASHBOARDSearchContext context) {
            com.baomidou.mybatisplus.extension.plugins.pagination.Page<DYNADASHBOARD> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
            return new PageImpl<DYNADASHBOARD>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<DYNADASHBOARD> getDynadashboardByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<DYNADASHBOARD> getDynadashboardByEntities(List<DYNADASHBOARD> entities) {
        List ids =new ArrayList();
        for(DYNADASHBOARD entity : entities){
            Serializable id=entity.getDynadashboardid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




    public IDYNADASHBOARDService getProxyService() {
        return cn.ibizlab.util.security.SpringContextHolder.getBean(this.getClass());
    }
}



